@@ -241,7 +241,7 @@
 #define TLS_server_method                SSLv23_server_method
 #endif /* OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER) */
 
-#if OPENSSL_VERSION_NUMBER >= 0x10101000L
+#if OPENSSL_VERSION_NUMBER >= 0x10101000L && !defined(LIBRESSL_VERSION_NUMBER)
 #define HAVE_KEYLOG_CALLBACK
 #endif
 
@@ -318,6 +318,7 @@ struct tcn_ssl_ctxt_t {
     unsigned int    alpn_proto_len;
     int             alpn_selector_failure_behavior;
     /* End add from netty-tcnative */
+    int             no_ocsp_check;
 };
 
 #ifdef HAVE_SSL_CONF_CMD
@@ -326,6 +327,7 @@ typedef struct tcn_ssl_conf_ctxt_t tcn_ssl_conf_ctxt_t;
 struct tcn_ssl_conf_ctxt_t {
     apr_pool_t      *pool;
     SSL_CONF_CTX    *cctx;
+    int     no_ocsp_check;
 };
 #endif
 
@@ -395,7 +397,7 @@ int         SSL_callback_alpn_select_proto(SSL *, const unsigned char **, unsign
 void        SSL_callback_add_keylog(SSL_CTX *);
 #endif
 
-#if (OPENSSL_VERSION_NUMBER < 0x10100000L) && ! (defined(WIN32) || defined(WIN64))
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)) && ! (defined(WIN32) || defined(WIN64))
 unsigned long SSL_ERR_get(void);
 void SSL_ERR_clear(void);
 #else
