@@ -34,8 +34,6 @@ static apr_status_t ssl_context_cleanup(void *data)
     tcn_ssl_ctxt_t *c = (tcn_ssl_ctxt_t *)data;
     if (c) {
         int i;
-        if (c->crl)
-            X509_STORE_free(c->crl);
         c->crl = NULL;
         if (c->ctx)
             SSL_CTX_free(c->ctx);
@@ -99,12 +97,12 @@ int ssl_callback_ServerNameIndication(SSL *ssl, int *al, tcn_ssl_ctxt_t *c)
     jstring hostname;
     jlong original_ssl_context, new_ssl_context;
     tcn_ssl_ctxt_t *new_c;
-    
+
     // Continue only if the static method exists
     if (sni_java_callback == NULL) {
         return SSL_TLSEXT_ERR_OK;
     }
-    
+
     (*javavm)->AttachCurrentThread(javavm, (void **)&env, NULL);
 
     // Get the host name presented by the client
@@ -113,7 +111,7 @@ int ssl_callback_ServerNameIndication(SSL *ssl, int *al, tcn_ssl_ctxt_t *c)
     // Convert to Java compatible parameters ready for the method call
     hostname = (*env)->NewStringUTF(env, servername);
     original_ssl_context = P2J(c);
-    
+
     new_ssl_context = (*env)->CallStaticLongMethod(env,
                                                    ssl_context_class,
                                                    sni_java_callback,
@@ -230,6 +228,10 @@ TCN_IMPLEMENT_CALL(jlong, SSLContext, make)(TCN_STDARGS, jlong pool,
         goto init_failed;
     }
 
+#ifdef HAVE_KEYLOG_CALLBACK
+    SSL_callback_add_keylog(ctx);
+#endif
+
     c->protocol = protocol;
     c->mode     = mode;
     c->ctx      = ctx;
@@ -607,14 +609,12 @@ TCN_IMPLEMENT_CALL(jboolean, SSLContext, setCARevocation)(TCN_STDARGS, jlong ctx
         lookup = X509_STORE_add_lookup(c->crl, X509_LOOKUP_file());
         if (lookup == NULL) {
             ERR_error_string(SSL_ERR_get(), err);
-            X509_STORE_free(c->crl);
             c->crl = NULL;
             tcn_Throw(e, "Lookup failed for file %s (%s)", J2S(file), err);
             goto cleanup;
         }
         if (!X509_LOOKUP_load_file(lookup, J2S(file), X509_FILETYPE_PEM)) {
             ERR_error_string(SSL_ERR_get(), err);
-            X509_STORE_free(c->crl);
             c->crl = NULL;
             tcn_Throw(e, "Load failed for file %s (%s)", J2S(file), err);
             goto cleanup;
@@ -624,20 +624,18 @@ TCN_IMPLEMENT_CALL(jboolean, SSLContext, setCARevocation)(TCN_STDARGS, jlong ctx
         lookup = X509_STORE_add_lookup(c->crl, X509_LOOKUP_hash_dir());
         if (lookup == NULL) {
             ERR_error_string(SSL_ERR_get(), err);
-            X509_STORE_free(c->crl);
             c->crl = NULL;
             tcn_Throw(e, "Lookup failed for path %s (%s)", J2S(file), err);
             goto cleanup;
         }
         if (!X509_LOOKUP_add_dir(lookup, J2S(path), X509_FILETYPE_PEM)) {
             ERR_error_string(SSL_ERR_get(), err);
-            X509_STORE_free(c->crl);
             c->crl = NULL;
             tcn_Throw(e, "Load failed for path %s (%s)", J2S(file), err);
             goto cleanup;
         }
     }
-    X509_STORE_set_flags(c->store, X509_V_FLAG_CRL_CHECK | X509_V_FLAG_CRL_CHECK_ALL);
+    X509_STORE_set_flags(c->crl, X509_V_FLAG_CRL_CHECK | X509_V_FLAG_CRL_CHECK_ALL);
     rv = JNI_TRUE;
 cleanup:
     TCN_FREE_CSTRING(file);
@@ -745,7 +743,7 @@ TCN_IMPLEMENT_CALL(void, SSLContext, setTmpDH)(TCN_STDARGS, jlong ctx,
         tcn_Throw(e, "Error while configuring DH: no dh param file given");
         return;
     }
-    
+
     bio = BIO_new_file(J2S(file), "r");
     if (!bio) {
         char err[256];
@@ -773,7 +771,7 @@ TCN_IMPLEMENT_CALL(void, SSLContext, setTmpDH)(TCN_STDARGS, jlong ctx,
         TCN_FREE_CSTRING(file);
         return;
     }
-    
+
     DH_free(dh);
     TCN_FREE_CSTRING(file);
 }
@@ -789,7 +787,7 @@ TCN_IMPLEMENT_CALL(void, SSLContext, setTmpECDHByCurveName)(TCN_STDARGS, jlong c
     UNREFERENCED(o);
     TCN_ASSERT(ctx != 0);
     TCN_ASSERT(curveName);
-    
+
     /* First try to get curve by name */
     i = OBJ_sn2nid(J2S(curveName));
     if (!i) {
@@ -797,14 +795,14 @@ TCN_IMPLEMENT_CALL(void, SSLContext, setTmpECDHByCurveName)(TCN_STDARGS, jlong c
         TCN_FREE_CSTRING(curveName);
         return;
     }
-    
+
     ecdh = EC_KEY_new_by_curve_name(i);
     if (!ecdh) {
         tcn_Throw(e, "Can't configure elliptic curve: unknown curve name %s", J2S(curveName));
         TCN_FREE_CSTRING(curveName);
         return;
     }
-    
+
     /* Setting found curve to context */
     if (1 != SSL_CTX_set_tmp_ecdh(c->ctx, ecdh)) {
         char err[256];
@@ -1264,7 +1262,7 @@ TCN_IMPLEMENT_CALL(jboolean, SSLContext, addChainCertificateRaw)(TCN_STDARGS, jl
         ERR_error_string(SSL_ERR_get(), err);
         tcn_Throw(e, "Error reading certificate (%s)", err);
         rv = JNI_FALSE;
-#if defined(LIBRESSL_VERSION_NUMBER)
+#if defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER < 0x20901000L
     } else {
         tcn_Throw(e, "Unable to use Java keystores with LibreSSL");
 #else
